function [price, W, exercised] = pricePutByHeatEquationImplicit( ...
    K, T, S0, r, sigma, N, M, nSds, american )
%EULERFORCALL Use the implicit method to price a put option
%   by solving the heat equation in negative time
%   N is the number of time steps
%   M is the number of steps in the x direction
%   nSds is the number of standard deviations either side of S0 to use
%        when defining the boundary of the grid.
%
%   We make the coordinate changes x=-(r-0.5 sigma^2)t + log S
%                                  W=e^{-rt} V
%   to transform the BS PDE into the heat equation in negative time
assert( rem(M,2)==0); % M must be even

x0 = log( S0 );
xMin = x0 - nSds*sqrt(T)*sigma;
xMax = x0 + nSds*sqrt(T)*sigma;

dt = T/N;
dx = (xMax-xMin)/M;

iMin = 1;
iMax = N+1;
jMin = 1;
jMax = M+1;
x = (xMin:dx:xMax)';
t = (0:dt:T);

lambda = 0.5*sigma^2 * dt/(dx)^2;

% Use boundary condition to create vector currW
currW=max(exp(-r*T)*K-exp(-0.5 *sigma^2 * T + x),0);
A = createTridiagonal( [0 ; -lambda*ones(M-1,1) ; 0], ...
                       [1 ; (1+2*lambda)*ones(M-1,1) ; 1], ...
                       [0 ; -lambda*ones(M-1,1) ; 0] );
    
bottom = exp(-r*T)*K- exp(-0.5*sigma^2 * t + x(jMin));               
top=zeros(1,N+1);                  


exercised = zeros(N+1,M+1);
W = zeros(N+1, M+1);
W(iMax,:)=currW;
for i=iMax-1:-1:iMin
    wIPlus1 = [ bottom(i); currW((jMin+1):(jMax-1)); top(i) ];
    if (american)
        % e = immediate exercise value
        e = max(exp(-r*t(i))*K-exp(-0.5 *sigma^2 * t(i) + x),0);
        b = A*e - wIPlus1;
        omega = 1.5;
        [xSol,ySol] = solveLCP(A, b, wIPlus1, omega, 10 );
        currW =  xSol + e;
        exercised(i,:)=currW<=(e);
    else
        currW = A \ wIPlus1;
    end
    W(i,:)=currW;
end
price = currW(jMin+M/2);

end



